use std::rc::Rc;

use super::{empty::Empty, symbol::Symbol, Environment, Node, NodeEnum, Precedence};

#[derive(Debug, Clone, PartialEq, PartialOrd)]
pub struct Assign {
    left: Symbol,
    right: Rc<NodeEnum>,
}

impl Node for Assign {
    fn evaluate(&self, env: &mut Environment) -> Result<Rc<NodeEnum>, String> {
        env.insert(self.left.get_value(), self.right.clone());
        Ok(Empty::EMPTY.clone())
    }

    fn as_string(&self, env: Option<&Environment>) -> String {
        format!(
            "{} := {}",
            self.left.as_string(env),
            self.right.as_string(env)
        )
    }

    fn precedence(&self) -> Precedence {
        Precedence::Assign
    }
}
